/* Тестовый скетч вывода украинского и русского алфавита на светодиодную матрицу с драйвером MAX7219.
   Для корректной работы необходимо заменить в папке библиотеки Adafruit_GFX_Library оригинальный файл
   glcdfont.c предоставленным, он имеет модифицированные кодировки сисволов. 
   В скетче иключен delay, уж очень я его не люблю...
   В примере использованы матрицы с общим анодом и схемой подключения в отличии от оригинальной конструкции, где
   покупные матрицы с общим катодом. Для их корректной работы требуется другое подключение, чем для матриц с общим катодом:
   MAX7219  MAX7219  2088AS  2088BS
   (номер)    (имя)    (ОК)    (ОА)
      3       DIG 4     1       6
      5       DIG 6     2       15     
      14      SEG A     3       2
      16      SEG B     4       7
      8       DIG 7     5       16
      23      SEG D     6       12
      10      DIG 5     7       11
      6       DIG 2     8       4
      2       DIG 0     9       13
      20      SEG C     10      1
      21      SEG E     11      8
      7       DIG 3     12      10
      22      SEG DP    13      5
      11      DIG 1     14      3
      15      SEG F     15      14
      17      SEG G     16      9

      19      VCC
      4,9     GND
      1       DIN
      12      CS (LOAD)
      13      CLK
      24      DOUT

    Несмотря на то, что геометрически матрицы с ОА в блоке расположены аналогично с матрицами с ОК, для их нормального
    функционирования необходим программный поворот на 90 градусов. Библиотека LedControl, обычно применяемая для 
    управления драйверами MAX7219, этого делать не умеет. Скетч - с использованием связки библиотек  Adafruit_GFX и Max72xxPanel.
    Немного про адресацию позиции точки на дисплее. Нулевым отсчетом (X=0 и Y=0) принят верхний левый
    угол слитного массива матриц при X - координата по горизонтали (в примере от 0 до 47), Y - координата по вертикали (от 0 до 7). 
    Исходя из этого расчитывать местоположение каждого элемена отображения, будь то точки или символа. Необходимо учесть, что 
    символы имеют ширину 5+1 (для пропуска). Всё вышеизложенное - для реализации информера: часы, термометр, будильники и т.д.
    Для покупных блоков матриц закомментировать строки 106-111, и вполне возможно - изменить 115-120.
*/    

#include <SPI.h>                                    // необходимые библиотеки, эта встроенная, две следующих по адресам
#include <Adafruit_GFX.h>                           // https://github.com/adafruit/Adafruit-GFX-Library
#include <Max72xxPanel.h>                           // https://github.com/markruys/arduino-Max72xxPanel

#define pinCS 3                                     // подключить CS к этому выводу, DIN к MOSI и CLK к SCK (cf http://arduino.cc/en/Reference/SPI )
#define numberOfHorizontalDisplays 6                // всего матриц по горизонтали
#define numberOfVerticalDisplays 1                  // всего матриц по вертикали

Max72xxPanel matrix = Max72xxPanel(pinCS, numberOfHorizontalDisplays, numberOfVerticalDisplays);

String tape = "";                                   // заготовка под строку

bool     fstr = 0;                                  // флаг состояния индикации (исходное или выключена)
uint32_t myTimer;                                   // переменная-таймер для установки временных интервалов
byte     lag = 20;                                  // переменная, определяет задержку прокрутки строки (мсек)

                 // перекодирование украинского и русского набора символов из UTF-8 в Windows-1251
                 // в одном скетче можно использовать как минимум 3 языка (включая английский)

String utf8cyr(String source)
{
  int i,k;
  String target;
  unsigned char n;
  char m[2] = { '0', '\0' };

  k = source.length(); i = 0;
  while (i < k) {
    n = source[i]; i++;
    if (n >= 0xC0) {
      switch (n) {
        case 0xD0: {                                // перекодировать 0 блок (прописные)
          n = source[i]; i++;                 
          if (n == 0x81) { n = 0xA8; break; }       // перекодировать букву Ё
          if (n == 0x84) { n = 0xAA; break; }       // перекодировать букву Є
          if (n == 0x86) { n = 0xB1; break; }       // перекодировать букву І                                                        
          if (n == 0x87) { n = 0xAF; break; }       // перекодировать букву Ї
          if (n >= 0x90 && n <= 0xBF) n = n + 0x2F; break; // перекодировать остальные буквы 0 блока
        }
        case 0xD1: {                                // перекодировать 1 блок (строчные)                   
          n = source[i]; i++;
          if (n == 0x91) { n = 0xB7; break; }       // перекодировать букву ё 
          if (n == 0x94) { n = 0xB9; break; }       // перекодировать букву є
          if (n == 0x96) { n = 0xB2; break; }       // перекодировать букву і             
          if (n == 0x97) { n = 0xBE; break; }       // перекодировать букву ї 
          if (n >= 0x80 && n <= 0x8F) n = n + 0x6F; break; // перекодировать остальные буквы 1 блока
        }
        case 0xD2: {                                // перекодировать 2 блок (всё вместе)
          n = source[i]; i++;
          if (n == 0x90) { n = 0xA5; break; }       // перекодировать букву Ґ          
          if (n == 0x91) { n = 0xB3; break; }       // перекодировать букву ґ             
        }
      }
    }
    m[0] = n; 
    target = target + String(m);
  }
return target;
}

void setup() {
  
  tape = utf8cyr("Max72xxPanel Ukr Rus Eng output test"); // этот текст выводится при включении
 
  matrix.setIntensity(5);                           // Use a value between 0 and 15 for brightness

  matrix.setPosition(0, 5, 0);                      // нулевой дисплей находится в <5, 0>
  matrix.setPosition(1, 4, 0);                      // первый дисплей находится в <4, 0>
  matrix.setPosition(2, 3, 0);                      // второй дисплей находится в <3, 0>
  matrix.setPosition(3, 2, 0);                      // третий дисплей находится в <2, 0>
  matrix.setPosition(4, 1, 0);                      // четвёртый дисплей находится в <1, 0>
  matrix.setPosition(5, 0, 0);                      // пятый дисплей находится в <0, 0>

  
// ...
// matrix.setRotation(0, 2);                        // нулевой дисплей находится в перевернутом положении
// matrix.setRotation(1, 2);                        // первый дисплей находится в перевернутом положении
// matrix.setRotation(2, 2);                        // второй дисплей находится в перевернутом положении
// matrix.setRotation(3, 2);                        // третий дисплей находится в перевернутом положении
// matrix.setRotation(4, 2);                        // четвёртый дисплей находится в перевернутом положении
// matrix.setRotation(5, 2);                        // пятый дисплей находится в перевернутом положении

// matrix.setRotation(3);
  regard(); 
}

void loop() {

  tape = utf8cyr("Ukr: АБВГҐДЕЄЖЗИІЇЙКЛМНОПРСТУФХЦЧШЩЬЮЯ абвгґдеєжзиіїйклмнопрстуфхцчшщьюя"); // это украинский
  regard();
  tape = utf8cyr("Rus: АБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯ абвгдеёжзийклмнопрстуфхцчшщъыьэюя"); // это русский
  regard();
  tape = utf8cyr("Eng: ABCDEFGHIJKLMNOPQRSTUVWXYZ abcdefghijklmnopqrstuvwxyz"); // это английский, utf8cyr и скобки можно опустить
  regard();
}

void regard(){                                       // функция приветственной строки и вывода на дисплей

  int spacer = 1;                                    // интервал между символами
  int width = 5 + spacer;                            // ширина: шрифт 5 пикселей + 1 интервал


  for ( int i = 0 ; i < width * tape.length() + matrix.width() - 1 - spacer;) { // цикл изменения позиции, инкрементация - в конце функции

    if (fstr == 0)                                   // если флаг опущен (светодиоды не светятся)
    {
      matrix.fillScreen(LOW);                        // очистить дисплей

      int letter = i / width;                        // переменная, определяющая смещение текста
      int x = (matrix.width() - 1) - i % width;      // расчитать начальную координату вывода по горизонтали
      int y = (matrix.height() - 8) / 2;             // центрировать текст по вертикали

      while ( x + width - spacer >= 0 && letter >= 0 ) { // цикл создания буфера изображения
        if ( letter < tape.length() ) {
          matrix.drawChar(x, y, tape[letter], HIGH, LOW, 1);
        }
        letter--;
        x -= width;
      }
      matrix.write();                                 // отправить изображение на дисплей
      fstr = 1;                                       // поднять флаг
    }
    else                                              // иначе - если индикатор светится (флаг поднят), проверить, прошел-ли заданный интервал:
    {                                                 // считать прошедшее после последнего срабатывания время и записать в переменную timeLeft
      uint32_t timeLeft = millis() - myTimer;
      if (timeLeft >= lag)                            // если прошло времени более заданного
      {
        myTimer += lag * (timeLeft / lag);            // провести коррекцию
        fstr = 0;                                     // флаг опустить
        i++;                                          // инкрементировать позицию 
      }
    }
  }
}
